"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExceljsConvertion = void 0;
const exceljs_1 = __importDefault(require("exceljs"));
const tree_1 = require("../formatter/tree");
const custom_1 = require("../formatter/custom");
const default_1 = require("../formatter/default");
class ExceljsConvertion {
    constructor() { }
    /**
     * This method help to find the data format and calls respective method, then return formatted data
     * @param {*} dataFormat
     * @param {*} data
     * @returns {*}
     */
    formattingData(dataFormat, data, headerName) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (dataFormat) {
                case 'default':
                    yield default_1.defaultFormat.startFormat(data);
                    return yield default_1.defaultFormat.getFormat();
                case 'custom':
                    yield custom_1.customFormat.startFormat(data);
                    return yield custom_1.customFormat.getFormat();
                case 'tree':
                    yield tree_1.treeFormat.startFormat(data, headerName);
                    return yield tree_1.treeFormat.getFormat();
                // return treeFormat.formatTree(data, headerName);
                default:
                    throw new Error('dataFormat must mention in config');
            }
        });
    }
    initProcess(config, data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let { sheetName, headerName, headerStyle, dataFormat } = config;
                let workbook = new exceljs_1.default.Workbook();
                let sheet = workbook.addWorksheet(sheetName);
                let { header, cellData } = yield this.formattingData(dataFormat, data, headerName);
                // Remove Duplicate from header, key should be unique
                let jsonObj = header.map(JSON.stringify);
                let uniqSet = new Set(jsonObj);
                let rmDupl = Array.from(uniqSet).map((elem) => JSON.parse(elem));
                sheet.columns = rmDupl;
                cellData.map((rowData) => sheet.addRow(rowData));
                if (dataFormat != 'default') {
                    sheet.eachRow((row, rowNum) => {
                        row.eachCell((cell, colNum) => {
                            cell.alignment = { wrapText: true };
                            if (rowNum > 1) {
                                if (cell.value.data.style) {
                                    cell.font = cell.value.data.style.font;
                                    cell.fill = cell.value.data.style.fill;
                                }
                                if (cell.value.type == 'link') {
                                    cell.value = {
                                        text: cell.value.data.text,
                                        hyperlink: cell.value.data.hyperLink,
                                        tooltip: cell.value.data.toolTip
                                    };
                                }
                                else {
                                    cell.value = cell.value.data.text;
                                }
                            }
                        });
                    });
                }
                sheet.getRow(1).fill = headerStyle.fill;
                sheet.getRow(1).font = headerStyle.font;
                workbook.creator = config.creator;
                workbook.lastModifiedBy = config.lastModifiedBy;
                workbook.created = new Date();
                return workbook;
                // workbook.xlsx.writeFile('output/test.xlsx');
            }
            catch (err) {
                throw new Error(err.message);
            }
        });
    }
}
exports.ExceljsConvertion = ExceljsConvertion;
// export const excelJsConvertion = new ExceljsConvertion();
